@@ -5,7 +5,7 @@
    driver classes/backends. All virtual functions have to be implemented by
    the specific driver class. See drvSAMPL.cpp
   
-   Copyright (C) 1993 - 2012 Wolfgang Glunz, wglunz35_AT_pstoedit.net
+   Copyright (C) 1993 - 2023 Wolfgang Glunz, wglunz35_AT_pstoedit.net
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -39,11 +39,15 @@
 #include "cppcomp.h"
 #endif
 
+#ifdef OS_WIN32_WCE
+#include "WinCEAdapter.h"
+#endif
 
 #include I_fstream
 #include I_stdio
 #include I_stdlib
 #include I_string_h
+#include I_iostream
 USESTD
 
 #ifndef assert
@@ -54,9 +58,10 @@ USESTD
 #include "miscutil.h"
 #endif
 
+#include <vector>
 
 // for compatibility checking
-static const unsigned int drvbaseVersion = 108;
+static constexpr unsigned int drvbaseVersion = 108;
 // 101 introduced the driverOK function
 // 102 introduced the font optimization (lasttextinfo_)
 // 103 introduced the -ssp support and the virtual pathscanbemerged
@@ -66,43 +71,42 @@ static const unsigned int drvbaseVersion = 108;
 // 107 new driver descriptions -- added info about clipping
 // 108 new driver descriptions -- added info about driver options
 
-const unsigned int	maxFontNamesLength = 1000;
-const unsigned int	maxPoints    = 80000;	// twice the maximal number of points in a path
-const unsigned int  maxPages     = 10000;   // maximum number of pages - needed for the array of bounding boxes
-const unsigned int	maxElements  = maxPoints/2;
-const unsigned int	maxSegments  = maxPoints/2;// at least half of maxpoints (if we only have segments with one point)
-
+constexpr unsigned int  maxPages     = 10000;   // maximum number of pages - needed for the array of bounding boxes
 
 class DLLEXPORT Point
 {
 public:
 	Point(float x, float y) : x_(x),y_(y) {}
 	Point() : x_(0.0f), y_(0.0f) {}; // for arrays
-	float x_;
-	float y_;
+
+	float x() const { return x_; }
+	float y() const { return y_; }
+	void set_x(float f) { x_ = f; }
+	void set_y(float f) { y_ = f; }
 	bool operator==(const Point & p2) const { 
 		return (x_ == p2.x_) && (y_ == p2.y_); //lint !e777
 	}
-	Point operator+(const Point & p) const { return Point (x_+p.x_,y_+p.y_); }
-	const Point & operator+=(const Point & p) { x_+=p.x_; y_+=p.y_; return *this; }
-	Point operator*(float f) const { return Point (x_*f,y_*f); }
-
-#if 0
-	friend bool operator==(const Point & p1, const Point & p2) { 
-		return (p1.x_ == p2.x_) && (p1.y_ == p2.y_); //lint !e777
+	bool operator!=(const Point & p2) const { 
+		return !(*this == p2);
 	}
-#endif
-#ifdef  BUGGYGPP
-	Point transform(const float * matrix) const;
-#else
-	Point transform(const float matrix[6]) const;
-#endif
+	Point operator+(const Point & p) const { return Point (x_ + p.x_, y_ + p.y_); }
+	const Point & operator+=(const Point & p) { x_+=p.x_; y_+=p.y_; return *this; }
+	Point operator-(const Point & p) const { return Point (x_ - p.x_, y_ - p.y_); }
+	const Point& operator-=(const Point& p) { x_ -= p.x_; y_ -= p.y_; return *this; }
+	Point operator*(float f) const { return Point(x_ * f, y_ * f); }
+	const Point& operator*=(float f) { x_ *= f; y_ *= f; return *this; }
+	Point operator/(float f) const { return Point(x_ / f, y_ / f); }
+	const Point& operator/=(float f) { x_ /= f; y_ /= f; return *this; }
 
-	friend ostream & operator<<(ostream & out,const Point &p) {
+	Point transform(const float matrix[6]) const;
+
+	friend ostream & operator<<(ostream & out, const Point &p) {
 		return out << "x: " << p.x_ << " y: " << p.y_ ;
 	}
 
 private:
+	float x_;
+	float y_;
 };
 
 struct DLLEXPORT BBox // holds two points describing a Bounding Box 
@@ -122,8 +126,10 @@ public:
 
 static const char emptyDashPattern[] =  "[ ] 0.0";
 
-class basedrawingelement ; // forward
-class DriverDescription ;  // forward
+constexpr int char_code_of_space = 32;
+
+class basedrawingelement; // forward
+class DriverDescription ; // forward
 
 class       DLLEXPORT    drvbase 
     // = TITLE
@@ -151,15 +157,18 @@ public:
 	enum cliptype { clip , eoclip };
 	enum linetype { solid=0, dashed, dotted, dashdot, dashdotdot }; // corresponding to the CGM patterns
 	struct DLLEXPORT TextInfo {
-		float 		x;
-		float		y;
+		Point       p;
+		float x() const { return p.x(); }
+		float y() const { return p.y(); }
 		float		FontMatrix[6];
-		float 		x_end; // pen coordinates after show in PostScript
-		float		y_end; // 
+		Point 		p_end; // pen coordinates after show in PostScript
+		float x_end() const { return p_end.x(); }
+		float y_end() const { return p_end.y(); }
 		RSString 	thetext;
 		RSString	glyphnames;
 		bool		is_non_standard_font;
 		RSString    currentFontName;
+		RSString    currentFontUnmappedName;
 		RSString    currentFontFamilyName;
 		RSString    currentFontFullName;
 		RSString    currentFontWeight;
@@ -191,10 +200,9 @@ public:
 					( currentB == cmp.currentB ); //lint !e777 // testing floats for ==
 		}
 		TextInfo() :
-			x(0.0f),
-			y(0.0f),
-			x_end(0.0f),
-			y_end(0.0f),
+			p(0.0f, 0.0f),
+			FontMatrix(),
+			p_end(0.0f, 0.0f),
 //			thetext(0),  // use standard ctor
 			is_non_standard_font(false),
 			currentFontSize(10.0f),
@@ -205,18 +213,18 @@ public:
 			colorName(""),
 			cx(0.0f),
 			cy(0.0f),
-			Char(32), // 32 means space
+			Char(char_code_of_space), 
 			ax(0.0f),
 			ay(0.0f), 
 			mappedtoIsoLatin1(true), 
 			remappedfont(false) {
-				for (int i = 0; i < 6 ; i++ ) FontMatrix[i] = 0.0f;
+			  // obsolete with new C++ - already done in init
+			  for (int i = 0; i < 6; i++) { FontMatrix[i] = 0.0f; }
 			}
-		~TextInfo() { }
-	private:
-		// declared but not defined
-		// const TextInfo & operator = (const TextInfo &); // default is ok
-		// TextInfo(const TextInfo &); // default is ok
+
+		~TextInfo() = default;
+        TextInfo(const TextInfo&) = default;
+		TextInfo& operator=(const TextInfo&) = default;
 	};
 
 private:
@@ -232,10 +240,10 @@ protected:
 		unsigned int    currentLineJoin;
 		float			currentMiterLimit;
 		unsigned int    nr;
-		basedrawingelement * * path; // a path is an array of pointers to basedrawingelements
+		std::vector<basedrawingelement *> path;
 		bool	 	isPolygon; // whether current path was closed via closepath or not
 		unsigned int	numberOfElementsInPath;
-		unsigned int	subpathoffset; // normally 0, but if subpathes are simulated
+		unsigned int	subpathoffset; // normally 0, but if subpaths are simulated
 									   // then this is set to the begin of the current subpath 
 									   // before show_path is executed
 		float           currentLineWidth;
@@ -270,19 +278,19 @@ protected:
 			pathWasMerged(false),
 			dashPattern(emptyDashPattern)
 			{
-			    path = new basedrawingelement *[maxElements];
 			}
 
 		virtual ~PathInfo() { // added virtual because of windows memory handling
 			// the path content is deleted by clear
 			clear();
-			delete [] path;
 		}
+		void addtopath(basedrawingelement * newelement,
+			           ostream & errf);
 		void clear();
 		void copyInfo(const PathInfo & p);
 			// copies the whole path state except the path array
 		void rearrange();
-            // rearrange subpaths for backends which don't support them 
+            // rearrange subpaths for backends which do not support them 
 	private:
 		// Inhibitors (declared, but not defined)
 		const PathInfo& operator=(const PathInfo&);
@@ -292,11 +300,11 @@ protected:
 //lint -esym(1712,SaveRestoreInfo) // no default ctor
 	class DLLEXPORT SaveRestoreInfo {
 	public:
-		unsigned int clippathlevel; // number of clippathes since opening (=save)
+		unsigned int clippathlevel; // number of clippaths since opening (=save)
 		unsigned int savelevel;	
 		SaveRestoreInfo * previous;	
 		SaveRestoreInfo * next;
-		SaveRestoreInfo(SaveRestoreInfo * parent) : clippathlevel(0), previous(parent), next(NIL) 
+		explicit SaveRestoreInfo(SaveRestoreInfo * parent) : clippathlevel(0), previous(parent), next(nullptr) 
 		{ 
 			if (parent) {
 				parent->next=this;
@@ -310,9 +318,10 @@ protected:
 public:
 	// = PUBLIC DATA
 
-	const DriverDescription * Pdriverdesc; // pointer to the derived class' driverdescription
+	const DriverDescription& driverdesc; // reference to the derived class' driverdescription
 
-	ProgramOptions* DOptions_ptr;
+	ProgramOptions* DOptions_ptr; // the driver specific options
+	const bool canDeleteDriverOptions;
 
 	PSImage 		imageInfo;
 
@@ -376,21 +385,22 @@ public:
 	// = CONSTRUCTION, DESTRUCTION AND COPYING
 
 	drvbase(
-		const char * driverOptions_p,
+		const char * driveroptions_p,
 		ostream & theoutStream,
 		ostream & theerrStream,		
 		const char* nameOfInputFile_p,
 		const char* nameOfOutputFile_p,
 		PsToEditOptions & globaloptions_p,
-		const DriverDescription * Pdriverdesc_p
+		ProgramOptions* driverOptions_p,
+		const class DriverDescription & driverdesc_p
 	); // constructor
 	virtual ~drvbase();  		// destructor
 
 	// = BACKEND GENERIC FUNCTIONS 
-        // These functions are not backend specific and shouldn't have to be
+        // These functions are not backend specific and should not have to be
         // changed for new backends
 
-	void		startup(bool merge);
+	void		startup(bool mergelines);
 
 	virtual void finalize(); 
 	// needed because base destructor will be called after derived destructor
@@ -401,6 +411,11 @@ public:
 	// under windows since the plugins are NOT and extension DLL
 	//
 
+	static bool use_fake_version_and_date; // for regression testing only
+	static const char * VersionString();
+	static void set_VersionString(const char * v);
+	static RSString DateString();
+
 	void		setdefaultFontName(const char * n) {defaultFontName = n;}
 
 	virtual bool textIsWorthToPrint(const RSString & thetext) const; // in the default implementation full blank strings are ignored
@@ -413,22 +428,22 @@ public:
 	void            setCurrentDeviceWidth(const float deviceWidth) 
 			{ currentDeviceWidth = deviceWidth; }
 
-	float           getScale() const { return 1.0f; }
+	static float           getScale() { return 1.0f; }
 
 	inline long l_transX			(float x) const	{
-		return (long)((x + x_offset) + .5);	// rounded long	
+		return static_cast <long>((x + x_offset) + 0.5f);	// rounded long	
 	}
 
 	inline long l_transY			(float y) const {
-		return (long)((-1.0f*y + y_offset) + .5);	// rounded long, mirrored
+		return static_cast <long>((-1.0f*y + y_offset) + 0.5f);	// rounded long, mirrored
 	}
 
 	inline int i_transX			(float x) const	{
-		return (int)((x + x_offset) + .5);	// rounded int	
+		return static_cast <int>((x + x_offset) + 0.5f);	// rounded int	
 	}
 
 	inline int i_transY			(float y) const {
-		return (int)((-1.0f*y + y_offset) + .5);	// rounded int, mirrored
+		return static_cast <int>((-1.0f*y + y_offset) + 0.5f);	// rounded int, mirrored
 	}
 
 	inline float f_transX			(float x) const	{
@@ -486,11 +501,18 @@ public:
 	void            setCurrentLineType(const linetype how) 
 			{ currentPath->currentLineType = how; }
 
+	const char * getLineTypeName() const {
+		static const char * lineTypeNames[] = {
+			"solid", "dashed", "dotted", "dashdot", "dashdotdot"
+		};
+		return lineTypeNames[currentPath->currentLineType];
+	}
+
 	void            setCurrentLineWidth(const float linewidth) 
 			{ currentPath->currentLineWidth = linewidth; }
 			
 	void 		setDash(const char * const dash)
-			{ currentPath->dashPattern.copy(dash); }
+			{ currentPath->dashPattern.assign(dash); }
 
 	void	 	setIsPolygon(bool what) { currentPath->isPolygon=what; } // whether current path was closed via closepath or not
 
@@ -515,7 +537,7 @@ public:
 
 	void		flushOutStanding( flushmode_t flushmode = flushall);
 
-	void		dumpRearrangedPathes(); // show the current subpathes after calling rearrange
+	void		dumpRearrangedPaths(); // show the current subpaths after calling rearrange
 
 	unsigned int nrOfSubpaths() const;
 
@@ -549,18 +571,18 @@ public:
 	const float *	getCurrentFontMatrix() const { return textInfo_.FontMatrix; }
 	void 		setCurrentFontMatrix(const float mat[6]) { for (unsigned short i = 0; i< 6; i++) textInfo_.FontMatrix[i] = mat[i]; }
 
-	// the push*Text methods set the textinfo_ member and then call the 
-	// showOrMergeText(textinfo_) ;
+	// the push*Text methods set the textInfo_ member and then call the 
+	// showOrMergeText(textInfo_) ;
 	void    	pushText(const size_t len,
 				        const char *const thetext,
 					const float x, 
 					const float y,
-					const char * const glyphnames=0);
+					const char * const glyphnames=nullptr);
 
 	void		pushHEXText(const char *const thetext, 
 					const float x, 
 					const float y,
-					const char * const glyphnames=0);
+					const char * const glyphnames=nullptr);
 
 	void		flushTextBuffer(bool useMergeBuffer); // flushes text from the text (merge) buffer 
 	void		showOrMergeText();
@@ -570,7 +592,7 @@ public:
 	// If a backend only deals with a special set of font names
 	// the following function must return a 0 terminated list
 	// of font names.
-	virtual const char * const * 	knownFontNames() const { return 0; }
+	virtual const char * const * 	knownFontNames() const { return nullptr; }
 
 	// The next functions are virtual with a default empty implementation
 
@@ -584,7 +606,7 @@ public:
 	// or it can just set the ctorOK to false.
 	virtual bool driverOK() const { return ctorOK; } // some  
 
-	// needed to check for pseude drivers which don't have a real backend
+	// needed to check for pseude drivers which do not have a real backend
 	// but instead just do the job in the gs frontend.
 	virtual bool withbackend() const { return true; }
 
@@ -608,9 +630,9 @@ protected:
 	float           fillG() const { return outputPath->fillG; }
 	float           fillB() const { return outputPath->fillB; }
 	const RSString & currentColorName() const { return outputPath->colorName; }
-	const char *    dashPattern() const { return outputPath->dashPattern.value(); }
-	float           currentR() const { return outputPath->fillR; } // backends that don't support merging 
-	float           currentG() const { return outputPath->fillG; } // don't need to differentiate and
+	const char *    dashPattern() const { return outputPath->dashPattern.c_str(); }
+	float           currentR() const { return outputPath->fillR; } // backends that do not support merging 
+	float           currentG() const { return outputPath->fillG; } // do not need to differentiate and
 	float           currentB() const { return outputPath->fillB; } // can use these functions.
 	void			add_to_page();
 
@@ -634,6 +656,10 @@ private:
 
 	virtual void    show_path() = 0;
 
+	void show_or_convert_path();
+
+	void simulate_fill();
+
 	// the next functions are virtual with default implementations
 
 	virtual void    show_text(const TextInfo & textinfo) ;
@@ -656,18 +682,20 @@ private:
 //lint -esym(1712,DashPattern) // no default ctor
 class DLLEXPORT DashPattern {
 public:
-	DashPattern(const char * patternAsSetDashString);
+	explicit DashPattern(const char * patternAsSetDashString);
 	~DashPattern();
 	const RSString dashString;
 	int nrOfEntries;
 	float * numbers;
 	float offset;
 
+private:
 	NOCOPYANDASSIGN(DashPattern)
+	DashPattern();
 };
 
 typedef const char * (*makeColorNameType)(float r, float g, float b);
-const unsigned int maxcolors = 10000 ; //  maximum number of colors 
+constexpr unsigned int maxcolors = 10000 ; //  maximum number of colors 
 
 //lint -esym(1712,ColorTable) // no default ctor
 class DLLEXPORT ColorTable 
@@ -693,17 +721,8 @@ private:
 
 
 
-#ifdef __TCPLUSPLUS__
-// turbo C++ has problems with enum for template parameters
-typedef unsigned int Dtype;
-const Dtype moveto = 1;
-const Dtype lineto = 2;
-const Dtype closepath = 3;
-const Dtype curveto = 4;
-#else
 enum  Dtype {moveto, lineto, closepath, curveto};
-#endif
-// closepath is only generated if backend supportes subpathes
+// closepath is only generated if backend supportes subpaths
 // curveto   is only generated if backend supportes it
 
 //lint -esym(1769,basedrawingelement)
@@ -713,38 +732,36 @@ class DLLEXPORT basedrawingelement
 {
 public:
 	// default ctor sufficient since no members anyway
-//	basedrawingelement(unsigned int size_p) /*: size(size_p) */ {}
+	basedrawingelement() = default;	
+	virtual ~basedrawingelement() = default;
 	virtual const Point &getPoint(unsigned int i) const = 0;
+	const Point& getLastPoint() const {
+		return getPoint(getNrOfPoints()-1);
+	}
 	virtual Dtype getType() const = 0;
-	friend ostream & operator<<(ostream & out,const basedrawingelement &elem);
-	bool operator==( const basedrawingelement & bd2) const;
+	friend ostream & operator<<(ostream & out, const basedrawingelement &elem);
+	bool operator==(const basedrawingelement& bd2) const;
 	virtual unsigned int getNrOfPoints() const = 0;
 	virtual basedrawingelement* clone() const = 0; // make a copy
 	// deleteyourself is needed because under Windows, the deletion
 	// of memory needs to be done by the same dll which did the allocation.
 	// this is not simply achieved if plugins are loaded as DLL.
 	virtual void deleteyourself() { delete this; } 
-	virtual ~basedrawingelement() {}
-private:
-//	const unsigned int size;
+	NOCOPYANDASSIGN(basedrawingelement)
 };
 
 
 inline void copyPoints(unsigned int nr, const Point src[], Point target[])
 {
 // needed because CenterLine cannot inline for loops
-	for (unsigned int i = 0 ; i < nr ; i++ ) target[i] = src[i]; 
+	for (size_t i = 0 ; i < nr ; i++ ) target[i] = src[i]; 
 }
 
 template <unsigned int nr, Dtype curtype>
 class drawingelement : public basedrawingelement
 {
 public:
-// CenterLine !!!!
-// "drvbase.h", line 455: sorry, not implemented: cannot expand inline function  drawingelement 
-//   <1 , 0 >::drawingelement__pt__19_XCUiL11XC5DtypeL10(Point*) with  for statement in inline
-
-	drawingelement(float x_1 = 0.0 ,float y_1 = 0.0 , float x_2 = 0.0, float y_2 = 0.0, float x_3 = 0.0, float y_3 = 0.0)
+	drawingelement(float x_1, float y_1, float x_2 = 0.0, float y_2 = 0.0, float x_3 = 0.0, float y_3 = 0.0)
 	: basedrawingelement()
 	{
 #if defined (__GNUG__) || defined (_MSC_VER) && _MSC_VER >= 1100
@@ -752,7 +769,7 @@ public:
 	copyPoints(nr,p,points);
 #else
 	// Turbo C++ hangs if the other solution is used.
-	// and the HP CC compiler doesn't like it either
+	// and the HP CC compiler does not like it either
 	// so use this for all compilers besides GNU and MS VC++
 	// This, however, is somewhat slower than the solution above
 	Point  * p = new Point[3];
@@ -765,12 +782,20 @@ public:
 
 	}
 
-	drawingelement(const Point p[])
+	explicit drawingelement(const Point p[])
 	: basedrawingelement()
 	{
-//	for (unsigned int i = 0 ; i < nr ; i++ ) points[i] = p[i]; 
 		copyPoints(nr,p,points);
 	}
+	// just for single point elements
+	explicit drawingelement(const Point & p)
+		: basedrawingelement()
+	{
+		// static_assert(nr == 1);
+		assert(nr == 1);
+		points[0] = p;
+	}
+
 	drawingelement(const drawingelement<nr,curtype> & orig)
 	: basedrawingelement() //lint !e1724 // Argument to copy constructor for class drawingelement<<1>,<2>> should be a const reference
 	{ // copy ctor
@@ -792,55 +817,62 @@ public:
 #endif
 		return points[i]; 
 	}
-	virtual Dtype getType() const 		     { return (Dtype) curtype; }
+	virtual Dtype getType() const 		     { return static_cast <Dtype>(curtype); }
 						// This cast (Dtype) is necessary
 						// to eliminate a compiler warning
 						// from the SparcCompiler 4.1.
 						// although curtype is of type Dtype
 	virtual unsigned int getNrOfPoints() const { return nr; }
 private:
-	Point points[(nr > 0) ? nr : (unsigned int)1]; //lint !e62 //Incompatible types (basic) for operator ':'
+	Point points[(nr > 0) ? nr : static_cast <unsigned int>(1)]; //lint !e62 //Incompatible types (basic) for operator ':'
+	drawingelement() = delete;
+	const drawingelement<nr, curtype>& operator=(const drawingelement<nr, curtype>& rhs) = delete;
 };
 
 
-// CenterLine !!!!
-// "drvbase.h", line 477: sorry, not implemented: cannot expand inline function  
-// drawingelement <3 , 3 >::drawingelement__pt__19_XCUiL13XC5DtypeL13(Point*) with  for statement in inline
-
-#if 0
-template <unsigned int nr, Dtype curtype>
-inline drawingelement<nr,curtype>::drawingelement(Point p[]) 
-	: basedrawingelement()
-{
-	for (unsigned int i = 0 ; i < nr ; i++ ) points[i] = p[i]; 
-}
-#endif
-
 typedef drawingelement<(unsigned int) 1,moveto>  	Moveto;
 typedef drawingelement<(unsigned int) 1,lineto> 	Lineto;
-typedef drawingelement<(unsigned int) 0,closepath>  	Closepath;
+typedef drawingelement<(unsigned int) 1,closepath> 	Closepath;
 typedef drawingelement<(unsigned int) 3,curveto> 	Curveto;
 
 
 #define derivedConstructor(Class)			\
 	Class(const char * driveroptions_p, 	\
-	       ostream & theoutStream, 			\
-	       ostream & theerrStream, 			\
-		   const char* nameOfInputFile_p,	\
-	       const char* nameOfOutputFile_p,	\
-		   PsToEditOptions & globaloptions_p, /* non const because driver can also add options */ 		\
-		   const DriverDescription * descptr)	
+              ostream & theoutStream, 			\
+              ostream & theerrStream, 			\
+              const char* nameOfInputFile_p,	\
+              const char* nameOfOutputFile_p,	\
+              PsToEditOptions& globaloptions_p, /* non const because driver can also add options */ 		\
+              ProgramOptions* driverOptions_p, \
+              const class DriverDescription & descref)	
+
+// use of static_cast instead of dynamic_cast, because some tools complain about problems then since
+// in theory dynamic_cast could return 0
+// Note: clang tidy wants dynamic_cast, but ignoring that wish.
+#define constructBase drvbase(driveroptions_p,theoutStream,theerrStream,nameOfInputFile_p,nameOfOutputFile_p,globaloptions_p,driverOptions_p,descref), \
+                      options(static_cast<DriverOptions*>(DOptions_ptr))
+//Note: 
+//room for improvement, but would need wider changes:
+//Class containing option descriptions is a local one in each driver class
+//But we need an instance of this already in drvbase ctor because we parse the options there 
+//and want to store the values.
+//So drvbase ctor creates the option object via createDriveroptions even before the derived driver class
+//is created completely.
+//Note: an "option" object is also needed/created for help output without creating a derived driver object.
+//Instead of adding the Description of Driver option in to the derived driver class
+//we would need a new class, e.g. derived from driverdescription<T> and add the options there.
+//But that is a wider change involving many source files.
+
 
-#define constructBase drvbase(driveroptions_p,theoutStream,theerrStream,nameOfInputFile_p,nameOfOutputFile_p,globaloptions_p,descptr), options((DriverOptions*)DOptions_ptr)
 
 
 class DLLEXPORT DescriptionRegister
 {
 	enum {maxelems = 100 };
 public:
-	DescriptionRegister() :ind(0) { 
-		for (int i = 0; i < maxelems; i++) rp[i] = 0; 
-	//	cout << " R constructed " << (void *) this << endl;
+	DescriptionRegister() :rp(), ind(0) { 
+		// obsolete with new C++ - already done in init
+		for (int i = 0; i < maxelems; i++) rp[i] = nullptr; 
 	}
 #if 0
 	// removed - since otherwise one gets a runtime error when the .so is unloaded 
@@ -855,6 +887,7 @@ public:
 	void registerDriver(DriverDescription* xp);
 	void mergeRegister(ostream & out,const DescriptionRegister & src,const char * filename);
 	void explainformats(ostream & out,bool withdetails=false) const;
+	void listdrivers(ostream &out) const;
 	const DriverDescription * getDriverDescForName(const char * drivername) const;
 	const DriverDescription * getDriverDescForSuffix(const char * suffix) const;
 
@@ -868,60 +901,36 @@ private:
 	NOCOPYANDASSIGN(DescriptionRegister)
 };
 
-//extern DLLEXPORT DescriptionRegister* globalRp;
 extern "C" DLLEXPORT DescriptionRegister * getglobalRp(void);
 
-//extern __declspec ( dllexport) "C" {
-//not needed // DescriptionRegister* getglobalRp();
 typedef DescriptionRegister* (*getglobalRpFuncPtr)(void);
-//}
-
-//class Rinit 
-//{
-//public:
-//	Rinit() { if (!globalRp) {globalRp = new DescriptionRegister; ref = 1 ; } else { ref++;} }
-//	~Rinit() { ref--; if (ref == 0) delete globalRp; }
-//
-//private:
-//	static	int ref;
-//};
-
-//static Rinit Rinit_var;
-
-//now in drvdesc.h typedef bool (*checkfuncptr)();
-// static bool nocheck() { return true; }
 
 typedef bool (*checkfuncptr)(void);
 class drvbase;
 
 struct OptionDescription {
-	OptionDescription(const char * n = 0, const char * p = 0, const char * d = 0) :Name(n), Parameter(p), Description(d) {}
-	const char * const Name;
-	const char * const Parameter;   // e.g. "String" or "numeric", or 0 (implicitly a boolean option then (no argument)
-	const char * const Description; // 
+	OptionDescription(const char * n, const char * p, const char * d) :Name(n), Parameter(p), Description(d) {}
+	const char * const Name = nullptr;
+	const char * const Parameter = nullptr;   // e.g. "String" or "numeric", or 0 (implicitly a boolean option then (no argument)
+	const char * const Description = nullptr; // 
 private:
-//	OptionDescription(const OptionDescription&);
-	const OptionDescription& operator=(const OptionDescription&);
 // no special copy ctor, assignment op or dtor needed since this class is NOT owner of the (static) strings.
+	OptionDescription() = delete;
+	OptionDescription(const OptionDescription&) = delete;
+	const OptionDescription& operator=(const OptionDescription&) = delete;
 };
 
-// An Array of OptionDescription is delimited by an element where Name is 0
-//FIXME const OptionDescription endofoptions(0,0,0);
-
-//FIXME const OptionDescription nodriverspecificoptions[] = {OptionDescription("driver has no further options",0,0),endofoptions};
-
-
 //lint -esym(1712,DriverDescription) // no default ctor
 class DLLEXPORT DriverDescription {
 public:
-	enum opentype {noopen, normalopen, binaryopen};
-	enum imageformat { noimage, png, bmp, eps, memoryeps }; // format to be used for transfer of raster images
+	enum class opentype {noopen, normalopen, binaryopen};
+	enum class imageformat { noimage, png, bmp, eps, memoryeps }; // format to be used for transfer of raster images
 
 	DriverDescription(const char * const s_name, 
 			const char * const short_expl, 
 			const char * const long_expl,
 			const char * const suffix_p, 
-			const bool 	backendSupportsSubPathes_p,
+			const bool 	backendSupportsSubPaths_p,
 			const bool 	backendSupportsCurveto_p,
 			const bool 	backendSupportsMerging_p, // merge a separate outline and filling of a polygon -> 1. element
 			const bool 	backendSupportsText_p,
@@ -931,22 +940,21 @@ public:
 			const bool	backendSupportsClipping_p,
 			const bool	nativedriver_p = true,
 			checkfuncptr checkfunc_p = 0);
-	virtual ~DriverDescription() {
-		//		symbolicname = NIL; // these are const
-		//		explanation= NIL;
-		//		suffix= NIL;
-		//		additionalInfo= NIL;
-	} //lint !e1540
+	virtual ~DriverDescription() = default;
 
 	virtual drvbase * CreateBackend (const char * const driveroptions_P,
 			 ostream & theoutStream, 
 			 ostream & theerrStream,   
 			 const char* const nameOfInputFile,
 		     const char* const nameOfOutputFile,
-			 PsToEditOptions & globaloptions_p
+			 PsToEditOptions & globaloptions_p,
+			 ProgramOptions * driverOptions_p
 			 ) const = 0;
 	virtual ProgramOptions * createDriverOptions() const = 0;
 
+	virtual size_t variants() const = 0;
+	virtual const DriverDescription * variant(size_t index) const = 0;
+
 	virtual unsigned int getdrvbaseVersion() const { return 0; } // this is only needed for the driverless backends (ps/dump/gs)
 
 	const char * additionalInfo() const;
@@ -956,7 +964,7 @@ public:
 	const char * const long_explanation;
 	const char * const suffix;
 
-	const bool 	backendSupportsSubPathes;
+	const bool 	backendSupportsSubPaths;
 	const bool 	backendSupportsCurveto;
 	const bool 	backendSupportsMerging; // merge a separate outline and filling of a polygon -> 1. element
 	const bool 	backendSupportsText;
@@ -965,7 +973,7 @@ public:
 	const bool	backendSupportsMultiplePages;
 	const bool	backendSupportsClipping;
 	const bool	nativedriver;
-	RSString filename; 
+	RSString 	filename; 
 	// where this driver is loaded from
 	// Note: formerly this was a RSString - but that caused problems under X64 / Windows
 	// it seems as constructing and deleting heap during start-up phase when not all DLLs are fully initialized
@@ -989,7 +997,7 @@ public:
 			const char * short_expl_p, 
 			const char * long_expl_p, 
 			const char * suffix_p, 
-			const bool 	backendSupportsSubPathes_p,
+			const bool 	backendSupportsSubPaths_p,
 			const bool 	backendSupportsCurveto_p,
 			const bool 	backendSupportsMerging_p, // merge a separate outline and filling of a polygon -> 1. element
 			const bool 	backendSupportsText_p,
@@ -1004,7 +1012,7 @@ public:
 			short_expl_p,
 			long_expl_p, 
 			suffix_p, 
-			backendSupportsSubPathes_p,
+			backendSupportsSubPaths_p,
 			backendSupportsCurveto_p,
 			backendSupportsMerging_p, 
 			backendSupportsText_p,
@@ -1015,17 +1023,20 @@ public:
 			nativedriver_p,
 			checkfunc_p
 			)
-		{}
-	drvbase * CreateBackend (
+	{
+	    instances().push_back(this);
+	}
+	virtual drvbase * CreateBackend (
 			const char * const driveroptions_P,
 		    ostream & theoutStream, 
 		    ostream & theerrStream,   
 			const char* const nameOfInputFile,
 	       	const char* const nameOfOutputFile,
-			PsToEditOptions & globaloptions_p /* non const because driver can also add arguments */
+			PsToEditOptions & globaloptions_p, /* non const because driver can also add arguments */
+		    ProgramOptions* driverOptions_p
 			 ) const
 	{ 
-		drvbase * backend = new T(driveroptions_P, theoutStream, theerrStream,nameOfInputFile,nameOfOutputFile, globaloptions_p,this); 
+		drvbase * backend = new T(driveroptions_P, theoutStream, theerrStream, nameOfInputFile, nameOfOutputFile, globaloptions_p, driverOptions_p , *this);
 		return backend;
 	} 
 
@@ -1035,17 +1046,32 @@ public:
 		return new typename T::DriverOptions;
 	}
 
-
-//	virtual void DeleteBackend(drvbase * & ptr) const { delete (T*) ptr; ptr = 0; }
+	//	virtual void DeleteBackend(drvbase * & ptr) const { delete (T*) ptr; ptr = 0; }
 	virtual unsigned int getdrvbaseVersion() const { return drvbaseVersion; }
 
+	static std::vector<const DriverDescriptionT<T> *> & instances() {
+	    static std::vector<const DriverDescriptionT<T> *> the_instances(0);
+	    return the_instances;
+	}
+
+	virtual size_t variants() const {
+		return instances().size();
+	}
+
+	virtual const DriverDescription* variant(size_t index) const {
+		if (index < instances().size()) {
+			return instances()[index];
+		} else {
+			return nullptr;
+		}
+	}
+
 private: 
-	// typedef DriverDescriptionT<T> SHORTNAME;
-	// NOCOPYANDASSIGN(SHORTNAME)
+
 	NOCOPYANDASSIGN(DriverDescriptionT<T>)
 };
 
-#if !( (defined (__GNUG__)  && (__GNUC__>=3) && defined (HAVESTL)) || defined (_MSC_VER) && (_MSC_VER >= 1300) )
+#if !( (defined (__GNUG__)  && (__GNUC__>=3) ) || defined (_MSC_VER) && (_MSC_VER >= 1300) && (_MSC_VER < 1900) )
 // 1300 is MSVC.net (7.0)
 // 1200 is MSVC 6.0
 //G++3.0 comes with a STL lib that includes a definition of min and max
@@ -1058,21 +1084,6 @@ private:
 #undef max
 #endif
 
-#ifndef min
-template <class T>
-inline T min(T x, T y)
-{
-	return (x<y) ? x:y;
-}
-#endif
-
-#ifndef max
-template <class T>
-inline T max(T x, T y)
-{
-	return (x>y) ? x:y;
-}
-#endif
 
 #endif
 
@@ -1092,17 +1103,10 @@ inline float bezpnt(float t, float z1, float z2, float z3, float z4)
 
 inline Point PointOnBezier(float t, const Point & p1, const Point & p2, const Point & p3, const Point & p4)
 {
-	return Point(bezpnt(t,p1.x_,p2.x_,p3.x_,p4.x_), bezpnt(t,p1.y_,p2.y_,p3.y_,p4.y_));
+	return Point(bezpnt(t,p1.x(),p2.x(),p3.x(),p4.x()), 
+				 bezpnt(t,p1.y(),p2.y(),p3.y(),p4.y()));
 }
 
 
-
-
 #endif
  
- 
- 
- 
- 
- 
- 
