@@ -2,7 +2,7 @@
    drvfuncs.h : This file is part of pstoedit
    This file contains the declarations of the functions, // each driver has to implement
 
-   Copyright (C) 1993 - 2012 Wolfgang Glunz, wglunz35_AT_pstoedit.net
+   Copyright (C) 1993 - 2023 Wolfgang Glunz, wglunz35_AT_pstoedit.net
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,14 +23,12 @@
 private: // yes these are private, library users should use the public interface
 	 // provided via drvbase
 
-	void open_page();
+	void open_page() override;
 
-	void close_page();
+	void close_page() override;
 
-	void show_path(); 
+	void show_path() override; 
 
-//	void show_text(const TextInfo & textInfo);
+//	void show_text(const TextInfo & textinfo);
 
 //	void show_rectangle(const float llx, const float lly, const float urx, const float ury);
-
-
