@@ -4,7 +4,7 @@
    miscutil.h : This file is part of pstoedit
    header declaring misc utility functions
 
-   Copyright (C) 1998 - 2019 Wolfgang Glunz, wglunz35_AT_pstoedit.net
+   Copyright (C) 1998 - 2023 Wolfgang Glunz, wglunz35_AT_pstoedit.net
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -43,15 +43,11 @@ USESTD
 #include <assert.h>
 #endif
 
-// used to eliminate compiler warnings about unused parameters
-inline void unused(const void * const) { }
-//lint -esym(522,unused)
-
 
 #if defined(_WIN32) || defined(__OS2__)
-const char directoryDelimiter = '\\';
+constexpr char directoryDelimiter = '\\';
 #else
-const char directoryDelimiter = '/';
+constexpr char directoryDelimiter = '/';
 #endif
 
 
@@ -62,12 +58,14 @@ const char directoryDelimiter = '/';
 
 inline char * cppstrndup(const char * const src, const size_t length, const size_t addon = 0 )
 {
-	const size_t lp1 = length+1;
-	char * const ret = new char[lp1 + addon];
-	for (unsigned int i = 0 ; i < lp1; i++)
+	assert(src);
+	const size_t lp1 = length+1; 
+	auto const ret = new char[lp1 + addon];
+	for (size_t i = 0 ; i < lp1; i++)
 	{
-			ret[i] = src[i];
+		ret[i] = src[i];
 	}
+	ret[length] = '\0';
 	return ret;
 
 }
@@ -106,136 +104,40 @@ private:
 #endif
 
 //#ifdef NOTYETFULLYBASEDONSTL
-#ifdef HAVE_STL
 #include <string>
 typedef std::string RSString;
 inline bool string_contains(const RSString & s, const RSString & substr) { return s.find(substr) !=RSString::npos; }
-
 #define DONOTIMPLEMENTRSSTRING
-
-#else
-// a very very simple resizing string
-// since STL is not yet available on all systems / compilers
-class DLLEXPORT RSString  {
-public:
-		
-	explicit RSString(const char * arg = 0);
-	explicit RSString(const char arg );
-	RSString(const char * arg , const size_t len);
-	RSString(const RSString & s);
-	virtual ~RSString();
-	const char *c_str() const { return content; }
-	char * value_to_change() { return content; }
-	size_t length() const { return stringlength; }
-	void assign(const char *src,const size_t len) ;
-	void assign(const char *src);
-	bool contains(const RSString & s) const;
-	// coverity[assign_returning_const]
-	const RSString & operator = (const RSString & rs) {
-		if (&rs != this) {
-			assign(rs.c_str(),rs.length());
-		}
-		return *this;
-	}
-	const RSString & operator = (const char * rs) {
-		assign(rs,strlen(rs));
-		return *this;
-	}
-
- 	RSString& operator+= (const RSString &rs);
-	
-	RSString& operator+= (const char * rs);
-	friend bool operator==(const RSString & ls,const RSString & rs);
-	friend bool operator!=(const RSString & ls,const RSString & rs); 
-
-//	bool operator<(const RSString & rs) const
-//	{	return strncmp(content,rs.content) < 0; }
-	char operator[](const size_t i) const
-	{	return content[i]; }
-	friend ostream & operator<<(ostream & out,const RSString &outstring)
-	{	if (outstring.content) out << outstring.content; return out; }
-
-private:
-	//
-	// we need to make the memory allocation/deallocation
-	// virtual in order to force that these are executed
-	// always in the same context (either main EXE or the
-	// loaded DLL)
-	// Since the plugins are not and MFC extension DLL memory
-	// allocation and deallocation cannot be mixed across the EXE
-	// and the DLL. It must be done in a symmetric way - if the memory
-	// is allocated by the DLL, it must be destroyed there as well.
-	//
-	virtual void clearContent();
-	// tell Flexelint: clearContent does cleanup
-	//lint -sem(RSString::clearContent,cleanup)
-	virtual char * newContent(size_t size);
-	char * content;
-	size_t allocatedLength;
-	size_t stringlength; // needed for storing binary strings including \0 
-};
-
-
-inline RSString  operator+ (const RSString & ls, const RSString &rs) 
-	{ RSString result(ls); result += rs; return result; }
-
-inline bool operator==(const RSString & ls, const RSString & rs) 
-	{ 	return ( (rs.stringlength == ls.stringlength ) && (strncmp(ls.content,rs.content,ls.stringlength) == 0) );	}
-inline bool operator!=(const RSString & ls, const RSString & rs)  
-	{ 	return !(ls==rs); }
-inline bool string_contains(const RSString & s, const RSString & substr) { return s.contains(substr);}
-#endif
+// FIXME
 
 
 inline bool strequal(const char * const s1, const char * const s2) { return (strcmp(s1,s2) == 0);}
 
 class Argv {
-	enum { maxargs=1000 };
+	static constexpr unsigned int maxargs = 1000;
 public:
-	unsigned int argc;
-// #define USE_RSSTRING 1
-#ifdef USE_RSSTRING
-	RSString argv[maxargs];
-#else
-	char * argv[maxargs];
-#endif
-
-	Argv() : argc(0) { for (unsigned int i = 0; i< (unsigned) maxargs; i++)  { argv[i] = 0; } }
+	unsigned int argc = 0;
+	char* argv[maxargs] = { nullptr };
+	Argv() = default;
 	~Argv() { clear(); }
 
 	void addarg(const char * const arg) { 
 		assert(argc<maxargs); //lint !e1776
 		if (argc < maxargs) {
-#ifdef USE_RSSTRING
-		   argv[argc] = RSString(arg);
-#else
 		   argv[argc] = cppstrdup(arg); 
-#endif
 		   argc++; 
 		}
 	}
 	void addarg(const RSString & arg) { 
-		assert(argc<maxargs); //lint !e1776
-		if (argc < maxargs) {
-#ifdef USE_RSSTRING
-		   argv[argc] = arg;
-#else
-		   argv[argc] = cppstrdup(arg.c_str()); 
-#endif
-		   argc++; 
-		}
+		addarg(arg.c_str());
 	}
 
 	unsigned int parseFromString(const char * const argstring);
 	
 	void clear() {
-#ifdef USE_RSSTRING
-#else
-		for (unsigned int i = 0; i< (unsigned) argc &&  i< (unsigned) maxargs ; i++) {
-
-			delete [] argv[i] ; argv[i]= 0; 
+		for (unsigned int i = 0; (i < argc) &&  (i < maxargs); i++) {
+			delete [] argv[i] ; argv[i]= nullptr; 
 		}
-#endif
 		argc = 0;
 	}
 	
@@ -246,15 +148,15 @@ DLLEXPORT ostream & operator <<(ostream & out, const Argv & a);
 
 DLLEXPORT bool fileExists (const char * filename);
 DLLEXPORT RSString full_qualified_tempnam(const char * pref);
-DLLEXPORT void convertBackSlashes(char* string);
+DLLEXPORT void convertBackSlashes(char* fileName);
 
 
 template <class T> 
 class DLLEXPORT Mapper {
 public:
-	Mapper() : firstEntry(0) {};
+	Mapper() : firstEntry(nullptr) {};
 	virtual ~Mapper() {
-		while (firstEntry != 0) {
+		while (firstEntry != nullptr) {
 			T * nextEntry = firstEntry->nextEntry;
 			delete firstEntry;
 			firstEntry=nextEntry;
@@ -332,7 +234,9 @@ DLLEXPORT void errorMessage(const char * errortext); // display an error message
 DLLEXPORT void copy_file(istream& infile,ostream& outfile) ;
 DLLEXPORT RSString getOutputFileNameFromPageNumber(const char * const outputFileTemplate, const RSString & pagenumberformatOption, unsigned int pagenumber);
 
-inline float pythagoras(const float x, const float y) { return sqrt( x*x + y*y); }
+inline float pythagoras(const float x, const float y) { 
+	return static_cast<float>(sqrt( x*x + y*y)); 
+}
 
 
 #endif
